@@ -15,8 +15,20 @@ from .exc import ArgumentError
 from . import functions  # NOQA
 from . import types  # NOQA
 
+import sqlalchemy
 from sqlalchemy import Table, event
-from sqlalchemy.sql import select, func, expression
+from sqlalchemy.sql import select, func, expression, text
+
+from packaging import version
+
+_SQLALCHEMY_VERSION_BEFORE_14 = version.parse(sqlalchemy.__version__) < version.parse("1.4")
+
+
+def _format_select_args(*args):
+    if _SQLALCHEMY_VERSION_BEFORE_14:
+        return [args]
+    else:
+        return args
 
 
 def _setup_ddl_event_listeners():
@@ -69,7 +81,7 @@ def _setup_ddl_event_listeners():
                     args = [table.schema] if table.schema else []
                     args.extend([table.name, c.name])
 
-                    stmt = select([getattr(func, drop_func)(*args)])
+                    stmt = select(*_format_select_args(getattr(func, drop_func)(*args)))
                     stmt = stmt.execution_options(autocommit=True)
                     bind.execute(stmt)
 
@@ -90,8 +102,10 @@ def _setup_ddl_event_listeners():
                     ])
                     if c.type.use_typmod is not None:
                         args.append(c.type.use_typmod)
+                    if bind.dialect.name == 'sqlite':
+                        args.append(not c.type.nullable)
 
-                    stmt = select([func.AddGeometryColumn(*args)])
+                    stmt = select(*_format_select_args(func.AddGeometryColumn(*args)))
                     stmt = stmt.execution_options(autocommit=True)
                     bind.execute(stmt)
 
@@ -99,36 +113,53 @@ def _setup_ddl_event_listeners():
                 if isinstance(c.type, (Geometry, Geography)) and \
                         c.type.spatial_index is True:
                     if bind.dialect.name == 'sqlite':
-                        stmt = select([func.CreateSpatialIndex(table.name, c.name)])
+                        stmt = select(*_format_select_args(func.CreateSpatialIndex(table.name,
+                                                                                   c.name)))
                         stmt = stmt.execution_options(autocommit=True)
                         bind.execute(stmt)
                     elif bind.dialect.name == 'postgresql':
-                        if table.schema:
-                            bind.execute('CREATE INDEX "idx_%s_%s" ON "%s"."%s" '
-                                         'USING GIST ("%s")' %
-                                         (table.name, c.name, table.schema,
-                                          table.name, c.name))
+
+                        index_name = '"idx_{}_{}"'.format(table.name, c.name)
+
+                        if c.type.use_N_D_index:
+                            gis_column = '"{}" gist_geometry_ops_nd'.format(c.name)
                         else:
-                            bind.execute('CREATE INDEX "idx_%s_%s" ON "%s" '
-                                         'USING GIST ("%s")' %
-                                         (table.name, c.name, table.name, c.name))
+                            gis_column = '"{}"'.format(c.name)
+
+                        if table.schema:
+                            table_name = '"{}"."{}"'.format(table.schema, table.name)
+                        else:
+                            table_name = '"{}"'.format(table.name)
+
+                        sql = "CREATE INDEX {} ON {} USING GIST ({})".format(index_name,
+                                                                             table_name,
+                                                                             gis_column)
+
+                        q = text(sql)
+
+                        bind.execute(q)
                     else:
                         raise ArgumentError('dialect {} is not supported'.format(bind.dialect.name))
 
+                if isinstance(c.type, (Geometry, Geography)) and c.type.spatial_index is False and \
+                        c.type.use_N_D_index is True:
+                    raise ArgumentError('Arg Error(use_N_D_index): spatial_index must be True')
+
                 # Add spatial indices for the Raster columns
                 #
                 # Note the use of ST_ConvexHull since most raster operators are
                 # based on the convex hull of the rasters.
                 if isinstance(c.type, Raster) and c.type.spatial_index is True:
                     if table.schema:
-                        bind.execute('CREATE INDEX "idx_%s_%s" ON "%s"."%s" '
-                                     'USING GIST (ST_ConvexHull("%s"))' %
-                                     (table.name, c.name, table.schema,
-                                      table.name, c.name))
+                        q = text('CREATE INDEX "idx_%s_%s" ON "%s"."%s" '
+                                 'USING GIST (ST_ConvexHull("%s"))' %
+                                 (table.name, c.name, table.schema,
+                                  table.name, c.name))
                     else:
-                        bind.execute('CREATE INDEX "idx_%s_%s" ON "%s" '
-                                     'USING GIST (ST_ConvexHull("%s"))' %
-                                     (table.name, c.name, table.name, c.name))
+                        q = text('CREATE INDEX "idx_%s_%s" ON "%s" '
+                                 'USING GIST (ST_ConvexHull("%s"))' %
+                                 (table.name, c.name, table.name, c.name))
+                    bind.execute(q)
 
         elif event == 'after-drop':
             # Restore original column list including managed Geometry columns
