@@ -94,6 +94,9 @@ class _GISType(UserDefinedType):
 
         Indicate if a spatial index should be created. Default is ``True``.
 
+    ``use_N_D_index``
+        Use the N-D index instead of the standard 2-D index.
+
     ``management``
 
         Indicate if the ``AddGeometryColumn`` and ``DropGeometryColumn``
@@ -129,10 +132,10 @@ class _GISType(UserDefinedType):
         geometry/geography columns. """
 
     def __init__(self, geometry_type='GEOMETRY', srid=-1, dimension=2,
-                 spatial_index=True, management=False, use_typmod=None,
-                 from_text=None, name=None):
+                 spatial_index=True, use_N_D_index=False, management=False, use_typmod=None,
+                 from_text=None, name=None, nullable=True):
         geometry_type, srid = self.check_ctor_args(
-            geometry_type, srid, dimension, management, use_typmod)
+            geometry_type, srid, dimension, management, use_typmod, nullable)
         self.geometry_type = geometry_type
         self.srid = srid
         if name is not None:
@@ -141,9 +144,11 @@ class _GISType(UserDefinedType):
             self.from_text = from_text
         self.dimension = dimension
         self.spatial_index = spatial_index
+        self.use_N_D_index = use_N_D_index
         self.management = management
         self.use_typmod = use_typmod
         self.extended = self.as_binary == 'ST_AsEWKB'
+        self.nullable = nullable
 
     def get_col_spec(self):
         if not self.geometry_type:
@@ -151,6 +156,7 @@ class _GISType(UserDefinedType):
         return '%s(%s,%d)' % (self.name, self.geometry_type, self.srid)
 
     def column_expression(self, col):
+        """Specific column_expression that automatically adds a conversion function"""
         return getattr(func, self.as_binary)(col, type_=self)
 
     def result_processor(self, dialect, coltype):
@@ -165,6 +171,7 @@ class _GISType(UserDefinedType):
         return process
 
     def bind_expression(self, bindvalue):
+        """Specific bind_expression that automatically adds a conversion function"""
         return getattr(func, self.from_text)(bindvalue, type_=self)
 
     def bind_processor(self, dialect):
@@ -196,7 +203,7 @@ class _GISType(UserDefinedType):
         return process
 
     @staticmethod
-    def check_ctor_args(geometry_type, srid, dimension, management, use_typmod):
+    def check_ctor_args(geometry_type, srid, dimension, management, use_typmod, nullable):
         try:
             srid = int(srid)
         except ValueError:
@@ -225,6 +232,11 @@ class _GISType(UserDefinedType):
 
         if use_typmod and not management:
             warnings.warn('use_typmod ignored when management is False')
+        if use_typmod is not None and not nullable:
+            raise ArgumentError(
+                'The "nullable" and "use_typmod" arguments can not be used together'
+            )
+
         return geometry_type, srid
 
 
