@@ -1,7 +1,7 @@
 from json import loads
 import os
-from pkg_resources import parse_version
 import pytest
+import re
 
 try:
     from psycopg2cffi import compat
@@ -11,28 +11,35 @@ else:
     compat.register()
     del compat
 
-from sqlalchemy import __version__ as SA_VERSION
+from pkg_resources import parse_version
 from sqlalchemy import create_engine
 from sqlalchemy import Table, MetaData, Column, Integer, String, bindparam
 from sqlalchemy.orm import sessionmaker
 from sqlalchemy.ext.declarative import declarative_base
-from sqlalchemy.engine import reflection
 from sqlalchemy.exc import DataError, IntegrityError, InternalError, ProgrammingError
 from sqlalchemy.sql import select, func
 from sqlalchemy.sql.expression import type_coerce
+from sqlalchemy import __version__ as SA_VERSION
 
 from geoalchemy2 import Geometry, Geography, Raster
 from geoalchemy2.elements import WKTElement, WKBElement, RasterElement
 from geoalchemy2.shape import from_shape
-
+from geoalchemy2.exc import ArgumentError
 from shapely.geometry import LineString, Point
 
-from . import skip_postgis1, skip_postgis2
+from . import skip_postgis1, skip_postgis2, skip_case_insensitivity, skip_pg12_sa1217
 
+SQLA_LT_2 = parse_version(SA_VERSION) <= parse_version("2")
+if SQLA_LT_2:
+    from sqlalchemy.engine.reflection import Inspector
+    get_inspector = Inspector.from_engine
+else:
+    from sqlalchemy import inspect as get_inspector
 
 engine = create_engine(
     os.environ.get('PYTEST_DB_URL', 'postgresql://gis:gis@localhost/gis'), echo=False)
 metadata = MetaData(engine)
+arg_metadata = MetaData(engine)
 Base = declarative_base(metadata=metadata)
 
 
@@ -76,6 +83,13 @@ class IndexTestWithSchema(Base):
     geom2 = Column(Geometry(geometry_type='POINT', srid=4326, management=True))
 
 
+class IndexTestWithNDIndex(Base):
+    __tablename__ = 'index_test_with_nd_index'
+    __table_args__ = {'schema': 'gis'}
+    id = Column(Integer, primary_key=True)
+    geom1 = Column(Geometry(geometry_type='POINTZ', dimension=3, use_N_D_index=True))
+
+
 class IndexTestWithoutSchema(Base):
     __tablename__ = 'indextestwithoutschema'
     id = Column(Integer, primary_key=True)
@@ -86,6 +100,9 @@ class IndexTestWithoutSchema(Base):
 session = sessionmaker(bind=engine)()
 
 postgis_version = session.execute(func.postgis_lib_version()).scalar()
+postgres_major_version = re.match(r"([0-9]*)\.([0-9]*).*", session.execute(
+    """SELECT current_setting('server_version');""").scalar()).group(1)
+
 if postgis_version.startswith('1.'):
     # With PostGIS 1.x the AddGeometryColumn and DropGeometryColumn
     # management functions should be used.
@@ -114,9 +131,10 @@ class TestIndex():
     def teardown(self):
         session.rollback()
         metadata.drop_all()
+        arg_metadata.drop_all()
 
     def test_index_with_schema(self):
-        inspector = reflection.Inspector.from_engine(engine)
+        inspector = get_inspector(engine)
         indices = inspector.get_indexes(IndexTestWithSchema.__tablename__, schema='gis')
         assert len(indices) == 2
         assert not indices[0].get('unique')
@@ -124,9 +142,56 @@ class TestIndex():
         assert not indices[1].get('unique')
         assert indices[1].get('column_names')[0] in (u'geom1', u'geom2')
 
+    def test_n_d_index(self):
+
+        engine.connect()
+        sql = """SELECT
+                    tablename,
+                    indexname,
+                    indexdef
+                FROM
+                    pg_indexes
+                WHERE
+                    tablename = 'index_test_with_nd_index'
+                ORDER BY
+                    tablename,
+                    indexname"""
+        r = engine.execute(sql)
+        results = r.fetchall()
+
+        for index in results:
+            if 'geom1' in index[1]:
+                nd_index = index[2]
+
+        index_type = nd_index.split("USING ", 1)[1]
+        assert index_type == 'gist (geom1 gist_geometry_ops_nd)'
+
+        inspector = get_inspector(engine)
+
+        indices = inspector.get_indexes(IndexTestWithNDIndex.__tablename__)
+        assert len(indices) == 1
+        assert not indices[0].get('unique')
+        assert indices[0].get('column_names')[0] in (u'geom1')
+
+    def test_n_d_index_argument_error(self):
+        BaseArgTest = declarative_base(metadata=arg_metadata)
+
+        class NDIndexArgErrorSchema(BaseArgTest):
+            __tablename__ = 'nd_index_error_arg'
+            __table_args__ = {'schema': 'gis'}
+            id = Column(Integer, primary_key=True)
+            geom1 = Column(Geometry(geometry_type='POINTZ',
+                                    dimension=3,
+                                    spatial_index=False,
+                                    use_N_D_index=True))
+
+        with pytest.raises(ArgumentError) as excinfo:
+            NDIndexArgErrorSchema.__table__.create(engine)
+        assert "Arg Error(use_N_D_index): spatial_index must be True" == excinfo.value.args[0]
+
     def test_index_without_schema(self):
-        inspector = reflection.Inspector.from_engine(engine)
-        indices = inspector.get_indexes(IndexTestWithSchema.__tablename__)
+        inspector = get_inspector(engine)
+        indices = inspector.get_indexes(IndexTestWithoutSchema.__tablename__)
         assert len(indices) == 2
         assert not indices[0].get('unique')
         assert indices[0].get('column_names')[0] in (u'geom1', u'geom2')
@@ -144,12 +209,13 @@ class TestTypMod():
         session.rollback()
         metadata.drop_all()
 
+    @skip_pg12_sa1217(postgres_major_version)
     def test_SummitConstraints(self):
         """ Make sure the geometry column of table Summit is created with
         `use_typmod=false` (explicit constraints are created).
          """
 
-        inspector = reflection.Inspector.from_engine(engine)
+        inspector = get_inspector(engine)
         constraints = inspector.get_check_constraints(
             Summit.__tablename__, schema='gis')
         assert len(constraints) == 3
@@ -272,7 +338,11 @@ class TestSelectBindParam():
 
     def test_select_bindparam(self):
         s = Lake.__table__.select().where(Lake.__table__.c.geom == bindparam('geom'))
-        results = self.conn.execute(s, geom='SRID=4326;LINESTRING(0 0,1 1)')
+        params = {"geom": "SRID=4326;LINESTRING(0 0,1 1)"}
+        if SQLA_LT_2:
+            results = self.conn.execute(s, **params)
+        else:
+            results = self.conn.execute(s, parameters=params)
         rows = results.fetchall()
 
         row = rows[0]
@@ -285,7 +355,11 @@ class TestSelectBindParam():
     def test_select_bindparam_WKBElement(self):
         s = Lake.__table__.select().where(Lake.__table__.c.geom == bindparam('geom'))
         wkbelement = from_shape(LineString([[0, 0], [1, 1]]), srid=4326)
-        results = self.conn.execute(s, geom=wkbelement)
+        params = {"geom": wkbelement}
+        if SQLA_LT_2:
+            results = self.conn.execute(s, **params)
+        else:
+            results = self.conn.execute(s, parameters=params)
         rows = results.fetchall()
 
         row = rows[0]
@@ -304,7 +378,11 @@ class TestSelectBindParam():
         assert geom.extended is True
 
         s = Lake.__table__.select().where(Lake.__table__.c.geom == bindparam('geom'))
-        results = self.conn.execute(s, geom=geom)
+        params = {"geom": geom}
+        if SQLA_LT_2:
+            results = self.conn.execute(s, **params)
+        else:
+            results = self.conn.execute(s, parameters=params)
         rows = results.fetchall()
 
         row = rows[0]
@@ -424,7 +502,11 @@ class TestUpdateORM():
 
         # Check what was updated in DB
         assert lake.geom is None
-        assert session.execute(select([Lake])).fetchall() == [(1, None)]
+        cols = [Lake.id, Lake.geom]
+        if SQLA_LT_2:
+            assert session.execute(select(cols)).fetchall() == [(1, None)]
+        else:
+            assert session.execute(select(*cols)).fetchall() == [(1, None)]
 
         # Reset geometry to initial value
         lake.geom = WKTElement(raw_wkt, srid=4326)
@@ -462,7 +544,11 @@ class TestUpdateORM():
 
         # Check what was updated in DB
         assert lake.geom is None
-        assert session.execute(select([Lake])).fetchall() == [(1, None)]
+        cols = [Lake.id, Lake.geom]
+        if SQLA_LT_2:
+            assert session.execute(select(cols)).fetchall() == [(1, None)]
+        else:
+            assert session.execute(select(*cols)).fetchall() == [(1, None)]
 
         # Reset geometry to initial value
         lake.geom = from_shape(shape, srid=4326)
@@ -524,7 +610,11 @@ class TestUpdateORM():
 
         # Check what was updated in DB
         assert o.rast is None
-        assert session.execute(select([Ocean])).fetchall() == [(1, None)]
+        cols = [Ocean.id, Ocean.rast]
+        if SQLA_LT_2:
+            assert session.execute(select(cols)).fetchall() == [(1, None)]
+        else:
+            assert session.execute(select(*cols)).fetchall() == [(1, None)]
 
         # Reset rast to initial value
         o.rast = RasterElement(rast_data)
@@ -582,6 +672,7 @@ class TestCallFunction():
 
     def teardown(self):
         session.rollback()
+        session.expunge_all()
         metadata.drop_all()
 
     def _create_one_lake(self):
@@ -771,9 +862,7 @@ class TestCallFunction():
             "properties": {"dummy_attr": 10, "id": 1}
         }
 
-    @pytest.mark.skipif(
-        parse_version(SA_VERSION) < parse_version("1.3.4"),
-        reason='Case-insensitivity is only available for sqlalchemy>=1.3.4')
+    @skip_case_insensitivity()
     def test_comparator_case_insensitivity(self):
         lake_id = self._create_one_lake()
 
@@ -810,7 +899,7 @@ class TestCallFunction():
         self._create_one_lake()
 
         s = select([func.ST_UnknownFunction(Lake.__table__.c.geom, 2)])
-        with pytest.raises(ProgrammingError):
+        with pytest.raises(ProgrammingError, match="ST_UnknownFunction"):
             session.execute(s)
 
     def test_unknown_function_element(self):
@@ -824,6 +913,13 @@ class TestCallFunction():
             # It would be better if it could fail because of a "UndefinedFunction" error
             session.execute(s)
 
+    def test_unknown_function_element_ORM(self):
+        lake_id = self._create_one_lake()
+        lake = session.query(Lake).get(lake_id)
+
+        with pytest.raises(AttributeError):
+            select([lake.geom.ST_UnknownFunction(2)])
+
 
 class TestReflection():
 
@@ -834,6 +930,7 @@ class TestReflection():
     def teardown(self):
         metadata.drop_all()
 
+    @skip_pg12_sa1217(postgres_major_version)
     def test_reflection(self):
         t = Table(
             'lake',
@@ -851,6 +948,7 @@ class TestReflection():
             assert type_.srid == 4326
 
     @skip_postgis1(postgis_version)
+    @skip_pg12_sa1217(postgres_major_version)
     def test_raster_reflection(self):
         t = Table('ocean', MetaData(), autoload=True, autoload_with=engine)
         type_ = t.c.rast.type
